/* StereoMatching.h ======================================================================*

*Important
===================================================
Raytrix Lenslet Converting Version 0.2

Copyright: ONLY Available for ACADEMIC and MPEG Standardization Purposes
Production: Nagoya University


*Description
===================================================
Given a set of Raw video captured by multi-focused plenoptic camera (i.e. Ratrix camera, https://raytrix.de/) 
and its microlens calibration data, the provided source code can render multi-view video.

This tool is implemented by:

Fujii Laboratory
Department of Information and Communication Engineering
Graduate School of Engineering
Nagoya University


*Term of Use
===================================================
Term of Use: ANY kind of academic publication, and report using this tool MUST refer to the following TWO references.

[1] Shu Fujita, Sho Mikawa, Mehrdad Teratani(Panahpourtehrani), Keita Takahashi, and Toshiaki Fujii 
“Extracting Multi-View Images from Multi-Focused Plenoptic Camera,” 
International Workshop on Advanced Image Technology (IWAIT)/International Forum on Medical Imaging in Asia (IFMIA), 
in SPIE Digital Library, Singapore, January 2019

[2] ] Mehrdad Teratani(Panahpourtehrani), Shu Fujita, Wenzhe Ouyang, Keita Takahashi, Toshiaki Fujii, 
"A 3-D Imaging System Using Multi-Focused Plenoptic Camera and Tensor Display," 
2018 International Conference on 3D Immersion (IC3D2018) in IEEE Xplore Digital Library, Brussels, Belgium, December 2018 

===================================================
* Class for generating raytrix multi-view image
*
* @file		RenderingRxMV > MultiViewImage.h
* @author	S. Mikawa and S. Fujita
* @date		2018/02/dd
*======================================================================*/

#pragma once
#include <opencv2/opencv.hpp>
#include "MicroLens.h"

using namespace std;
using namespace cv;

class RaytrixStereoMatching
{
private:
	MicroLens ml;
	Size blockSize;
	int minDisparity;
	int disparityNum;	
	int errorTruncate;
	int preFilterCap;
	Mat targetImage;
	Mat referenceImage;
	Mat lensMaskImage;
	Mat minCostMap;
	vector<Mat> DSI;

	enum { LEFT, RIGHT, LEFT_TOP, LEFT_BOTTOM, RIGHT_TOP, RIGHT_BOTTOM };

	void absDiffTruncateSSE(Mat& src1, Mat& src2, uchar thresh, Mat& destImage);
	void prefilterXSobel(Mat& srcImage, Mat& destImage, int ftzero);
	void shiftImage(InputArray srcImage, OutputArray destImage, const int hShift, const int vShift);
	void getMatchingCostSAD(Mat& targetImage, Mat& referenceImage, Mat& destImage, const int hShift, const int vShift);
	void getWTA(vector<Mat>& dsi, Mat& destImage);
	void generateLensMask(Mat& srcImage, Mat& maskImage, int erosionRadius);
	void initialization(Mat& srcImage);

public:

	RaytrixStereoMatching(MicroLens ml, int minDisparity, int maxDisparity, Size blockSize);

	// Core
	void operator()(Mat& srcImage, Mat& disparityImage);
};